/* 
 * title : xtery.c  -  battery status reporting
 * begin : 2003/02/13
 * make  : gcc -o xtery -I/usr/X11R6/include -L/usr/X11R6/lib -lX11 xtery.c -D `uname`
 * note  : 
 * 
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <errno.h>
#include <signal.h>

#ifdef FreeBSD
#include <fcntl.h>
#include <machine/apm_bios.h>
#endif

#include  "xtery.xbm"

#define  COLOR_HIGH       "Green"
#define  COLOR_LOW        "Yellow"
#define  COLOR_CRITICAL   "Red"
#define  COLOR_CHARGE     "Gray"
#define  WIN_WIDTH        130
#define  WIN_HEIGHT       20

/* abstructed power status structure */
struct POWER_STATE {
  int  ac;      /* AC line state */
  int  state;   /* battery state (Hi,Lo,Critical, Charging) */
  int  level;   /* battery level (in percentage) */
};

#ifdef FreeBSD
#define  APMDEV           "/dev/apm"
int  apm_fd;
struct apm_info  apm_info;
#endif

/*  $BJ8;zNs:n6HNN0h(B  */
char  str[128];
/*  $B2hA|99?74V3V(B  */
int  sleep_time = 5;
/*  $B%&%#%s%I%&4XO"(B  */
Display  *disp;
Window  win;
Pixmap  icon;
GC  gc;
Font  font;
XColor  col[4], exact;
XEvent  event;

/*  some convenient functions  */

int print_ok(char *str)
{
  fprintf(stderr, "%s\t... ok\n", str);
}

/* power state */
struct POWER_STATE  batt(void)
{
  struct POWER_STATE  ret;
  FILE  *fp;
#ifdef Linux
  fp = fopen("/proc/apm", "r");
  fscanf(fp, "%*f %*f %*x %x %x %*x %d", &ret.ac, &ret.state, &ret.level);
  fclose(fp);
#endif

#ifdef FreeBSD
  /* Read APM information. (AC line status, Battery status, etc.) */
  if (ioctl(apm_fd, APMIO_GETINFO, &apm_info) < 0) {
    fprintf(stderr, "ioctl(APMIO_GETINFO): %s\n", strerror(errno));
    exiting(1);
  }
  ret.ac    = apm_info.ai_acline;
  ret.level = apm_info.ai_batt_life;
  ret.state = apm_info.ai_batt_stat;
#endif
  return ret;
}


int drawWin()
{
  struct POWER_STATE  pow;

  /* read power state */
  pow = batt();
  /*  make window title  */
  sprintf(str, "xtery: %02d%%(%c)", pow.level, pow.ac == 1 ? 'A' : 'B');
  XStoreName(disp, win, str);
  XSetIconName(disp, win, str);

  /*  draw bar gauge  */
  XSetForeground(disp, gc, BlackPixel(disp, 0));
  XFillRectangle(disp, win, gc, 16, 4, 102, 12);
  XSetForeground(disp, gc, WhitePixel(disp, 0));
  XFillRectangle(disp, win, gc, 17, 5, 100, 10);
  XSetForeground(disp, gc, col[pow.state].pixel);
  XFillRectangle(disp, win, gc, 17, 5, pow.level, 10);
  /*  draw AC line / battery  */
  XCopyArea(disp, icon, win, gc, pow.ac*8, 0, 8, 16, 2, 2);

  XFlush(disp);
}

/* $B%7%0%J%kH/@8;~$N8F$S=P$74X?t(B */
void sig_draw()
{
  /* $B%"%i!<%`%7%0%J%k:F@_Dj(B */
  if (signal(SIGALRM, sig_draw) == SIG_ERR) {
    perror("signal");
    exit(1);
  }
  /* $B%2!<%8IA$-D>$7(B */
  drawWin();
  /* $B<!$N%"%i!<%`$r@_Dj(B */
  alarm(sleep_time);
}

/*  Ending  */
int  exiting(int st)
{
  XCloseDisplay(disp);
#ifdef FreeBSD
  close(apm_fd);
#endif
  exit(st);
}


/*  main  */

int main()
{
#ifdef FreeBSD
  /* Open APM device */
  if ((apm_fd = open(APMDEV, O_RDONLY)) < 0) {
    fprintf(stderr, "cannot open %s\n", APMDEV);
    exit(1);
  }
  print_ok("open APM device");
#endif

  /*  Map window  */
  disp = XOpenDisplay(NULL);
  win = XCreateSimpleWindow(disp, RootWindow(disp, 0), 
            800-WIN_WIDTH-5, 0, WIN_WIDTH, WIN_HEIGHT,
            2, BlackPixel(disp, 0), WhitePixel(disp, 0));
  XMapWindow(disp, win);
  XFlush(disp);
  print_ok("map window");

  gc = XCreateGC(disp, win, 0, 0);
  print_ok("create GC");

  icon = XCreatePixmapFromBitmapData(disp, win, icons_bits, 16, 16, BlackPixel(disp, 0), WhitePixel(disp, 0), DefaultDepth(disp, 0));
  print_ok("load icons");

  XAllocNamedColor(disp, DefaultColormap(disp, 0), COLOR_HIGH,
       &col[0], &exact);
  XAllocNamedColor(disp, DefaultColormap(disp, 0), COLOR_LOW,
       &col[1], &exact);
  XAllocNamedColor(disp, DefaultColormap(disp, 0), COLOR_CRITICAL,
       &col[2], &exact);
  XAllocNamedColor(disp, DefaultColormap(disp, 0), COLOR_CHARGE,
       &col[3], &exact);
  print_ok("alloc color");

  XSelectInput(disp, win, ButtonPressMask | StructureNotifyMask | ExposureMask);

  /* $B%"%i!<%`%7%0%J%k@_Dj(B */
  if (signal(SIGALRM, sig_draw) == SIG_ERR) {
    perror("signal");
    exit(1);
  }
  /* $B<!$N%"%i!<%`$r;XDj(B */
  alarm(sleep_time);

  /*  Main loop  */
  while (1) {
    XNextEvent(disp, &event);
    /* $B%^%&%9%\%?%s2!2<$G=*N;(B */
    if (event.type == ButtonPress) 
      break;
    /* $B%&%#%s%I%&Bg$-$5JQ99$G$b!"85$KLa$9(B */
    if (event.type == ConfigureNotify) 
      XResizeWindow(disp, win, WIN_WIDTH, WIN_HEIGHT);
    /* $B%&%#%s%I%&$,O*=P$7$?$i!":FIA2h(B */
    if (event.type == Expose && event.xany.window == win) {
      drawWin();
    }

  }

  exiting(0);

}
